@@ -50,7 +50,6 @@ $wp_file_descriptions = array(
  *
  * @since 1.5.0
  *
- * @uses _cleanup_header_comment
  * @uses $wp_file_descriptions
  * @param string $file Filesystem path or filename
  * @return string Description of file from $wp_file_descriptions or basename of $file if description doesn't exist
@@ -75,7 +74,6 @@ function get_file_description( $file ) {
  *
  * @since 1.5.0
  *
- * @uses get_option
  * @return string Full filesystem path to the root of the WordPress installation
  */
 function get_home_path() {
@@ -99,8 +97,8 @@ function get_home_path() {
  *
  * @since 2.6.0
  *
- * @param string $folder Full path to folder
- * @param int $levels (optional) Levels of folders to follow, Default: 100 (PHP Loop limit).
+ * @param string $folder Optional. Full path to folder. Default empty.
+ * @param int    $levels Optional. Levels of folders to follow, Default 100 (PHP Loop limit).
  * @return bool|array False on failure, Else array of files
  */
 function list_files( $folder = '', $levels = 100 ) {
@@ -139,8 +137,8 @@ function list_files( $folder = '', $levels = 100 ) {
  *
  * @since 2.6.0
  *
- * @param string $filename (optional) Filename to base the Unique file off
- * @param string $dir (optional) Directory to store the file in
+ * @param string $filename Optional. Filename to base the Unique file off. Default empty.
+ * @param string $dir      Optional. Directory to store the file in. Default empty.
  * @return string a writable filename
  */
 function wp_tempnam($filename = '', $dir = '') {
@@ -163,11 +161,9 @@ function wp_tempnam($filename = '', $dir = '') {
  *
  * @since 1.5.0
  *
- * @uses wp_die
- * @uses validate_file
  * @param string $file file the users is attempting to edit
  * @param array $allowed_files Array of allowed files to edit, $file must match an entry exactly
- * @return null
+ * @return string|null
  */
 function validate_file_to_edit( $file, $allowed_files = '' ) {
 	$code = validate_file( $file, $allowed_files );
@@ -212,10 +208,12 @@ function _wp_handle_upload( &$file, $overrides, $time, $action ) {
 	}
 
 	/**
-	 * The dynamic portion of the hook name, $action, refers to the post action.
+	 * Filter the data for a file before it is uploaded to WordPress.
 	 *
-	 * @since 2.9.0 as 'wp_handle_upload_prefilter'
-	 * @since 4.0.0 Converted to a dynamic hook with $action
+	 * The dynamic portion of the hook name, `$action`, refers to the post action.
+	 *
+	 * @since 2.9.0 as 'wp_handle_upload_prefilter'.
+	 * @since 4.0.0 Converted to a dynamic hook with `$action`.
 	 *
 	 * @param array $file An array of data for a single file.
 	 */
@@ -269,8 +267,6 @@ function _wp_handle_upload( &$file, $overrides, $time, $action ) {
 	$test_type = isset( $overrides['test_type'] ) ? $overrides['test_type'] : true;
 	$mimes = isset( $overrides['mimes'] ) ? $overrides['mimes'] : false;
 
-	$test_upload = isset( $overrides['test_upload'] ) ? $overrides['test_upload'] : true;
-
 	// A correct form post will pass this test.
 	if ( $test_form && ( ! isset( $_POST['action'] ) || ( $_POST['action'] != $action ) ) ) {
 		return call_user_func( $upload_error_handler, $file, __( 'Invalid form submission.' ) );
@@ -293,7 +289,7 @@ function _wp_handle_upload( &$file, $overrides, $time, $action ) {
 
 	// A properly uploaded file will pass this test. There should be no reason to override this one.
 	$test_uploaded_file = 'wp_handle_upload' === $action ? @ is_uploaded_file( $file['tmp_name'] ) : @ is_file( $file['tmp_name'] );
-	if ( $test_upload && ! $test_uploaded_file ) {
+	if ( ! $test_uploaded_file ) {
 		return call_user_func( $upload_error_handler, $file, __( 'Specified file failed upload test.' ) );
 	}
 
@@ -809,20 +805,25 @@ function copy_dir($from, $to, $skip_list = array() ) {
  * Initialises and connects the WordPress Filesystem Abstraction classes.
  * This function will include the chosen transport and attempt connecting.
  *
- * Plugins may add extra transports, And force WordPress to use them by returning the filename via the 'filesystem_method_file' filter.
+ * Plugins may add extra transports, And force WordPress to use them by returning
+ * the filename via the {@see 'filesystem_method_file'} filter.
  *
  * @since 2.5.0
  *
- * @param array $args (optional) Connection args, These are passed directly to the WP_Filesystem_*() classes.
- * @param string $context (optional) Context for get_filesystem_method(), See function declaration for more information.
- * @return boolean false on failure, true on success
+ * @param array  $args                         Optional. Connection args, These are passed directly to
+ *                                             the `WP_Filesystem_*()` classes. Default false.
+ * @param string $context                      Optional. Context for {@see get_filesystem_method()}.
+ *                                             Default false.
+ * @param bool   $allow_relaxed_file_ownership Optional. Whether to allow Group/World writable.
+ *                                             Default false.
+ * @return null|boolean false on failure, true on success.
  */
-function WP_Filesystem( $args = false, $context = false ) {
+function WP_Filesystem( $args = false, $context = false, $allow_relaxed_file_ownership = false ) {
 	global $wp_filesystem;
 
 	require_once(ABSPATH . 'wp-admin/includes/class-wp-filesystem-base.php');
 
-	$method = get_filesystem_method($args, $context);
+	$method = get_filesystem_method( $args, $context, $allow_relaxed_file_ownership );
 
 	if ( ! $method )
 		return false;
@@ -873,37 +874,68 @@ function WP_Filesystem( $args = false, $context = false ) {
 
 /**
  * Determines which Filesystem Method to use.
- * The priority of the Transports are: Direct, SSH2, FTP PHP Extension, FTP Sockets (Via Sockets class, or fsockopen())
+ *
+ * The priority of the Transports are: Direct, SSH2, FTP PHP Extension,
+ * FTP Sockets (Via Sockets class, or `fsockopen()`).
  *
  * Note that the return value of this function can be overridden in 2 ways
- *  - By defining FS_METHOD in your <code>wp-config.php</code> file
+ *
+ *  - By defining FS_METHOD in your `wp-config.php` file
  *  - By using the filesystem_method filter
- * Valid values for these are: 'direct', 'ssh2', 'ftpext' or 'ftpsockets'
- * Plugins may also define a custom transport handler, See the WP_Filesystem function for more information.
+ *
+ * Valid values for these are: 'direct', 'ssh2', 'ftpext' or 'ftpsockets'.
+ *
+ * Plugins may also define a custom transport handler, See the WP_Filesystem
+ * function for more information.
  *
  * @since 2.5.0
  *
+ * @todo Properly mark arguments as optional.
+ *
  * @param array $args Connection details.
  * @param string $context Full path to the directory that is tested for being writable.
+ * @param bool $allow_relaxed_file_ownership Whether to allow Group/World writable.
  * @return string The transport to use, see description for valid return values.
  */
-function get_filesystem_method($args = array(), $context = false) {
+function get_filesystem_method( $args = array(), $context = false, $allow_relaxed_file_ownership = false ) {
 	$method = defined('FS_METHOD') ? FS_METHOD : false; // Please ensure that this is either 'direct', 'ssh2', 'ftpext' or 'ftpsockets'
 
-	if ( ! $method && function_exists('getmyuid') && function_exists('fileowner') ){
-		if ( !$context )
-			$context = WP_CONTENT_DIR;
+	if ( ! $context ) {
+		$context = WP_CONTENT_DIR;
+	}
 
-		// If the directory doesn't exist (wp-content/languages) then use the parent directory as we'll create it.
-		if ( WP_LANG_DIR == $context && ! is_dir( $context ) )
-			$context = dirname( $context );
+	// If the directory doesn't exist (wp-content/languages) then use the parent directory as we'll create it.
+	if ( WP_LANG_DIR == $context && ! is_dir( $context ) ) {
+		$context = dirname( $context );
+	}
+
+	$context = trailingslashit( $context );
+
+	if ( ! $method ) {
 
-		$context = trailingslashit($context);
 		$temp_file_name = $context . 'temp-write-test-' . time();
 		$temp_handle = @fopen($temp_file_name, 'w');
 		if ( $temp_handle ) {
-			if ( getmyuid() == @fileowner($temp_file_name) )
+
+			// Attempt to determine the file owner of the WordPress files, and that of newly created files
+			$wp_file_owner = $temp_file_owner = false;
+			if ( function_exists('fileowner') ) {
+				$wp_file_owner = @fileowner( __FILE__ );
+				$temp_file_owner = @fileowner( $temp_file_name );
+			}
+
+			if ( $wp_file_owner !== false && $wp_file_owner === $temp_file_owner ) {
+				// WordPress is creating files as the same owner as the WordPress files, 
+				// this means it's safe to modify & create new files via PHP.
 				$method = 'direct';
+				$GLOBALS['_wp_filesystem_direct_method'] = 'file_owner';
+			} else if ( $allow_relaxed_file_ownership ) {
+				// The $context directory is writable, and $allow_relaxed_file_ownership is set, this means we can modify files
+				// safely in this directory. This mode doesn't create new files, only alter existing ones.
+				$method = 'direct';
+				$GLOBALS['_wp_filesystem_direct_method'] = 'relaxed_ownership';
+			}
+
 			@fclose($temp_handle);
 			@unlink($temp_file_name);
 		}
@@ -918,30 +950,39 @@ function get_filesystem_method($args = array(), $context = false) {
 	 *
 	 * @since 2.6.0
 	 *
-	 * @param string $method Filesystem method to return.
-	 * @param array  $args   An array of connection details for the method.
+	 * @param string $method  Filesystem method to return.
+	 * @param array  $args    An array of connection details for the method.
+	 * @param string $context Full path to the directory that is tested for being writable.
+	 * @param bool   $allow_relaxed_file_ownership Whether to allow Group/World writable.
 	 */
-	return apply_filters( 'filesystem_method', $method, $args );
+	return apply_filters( 'filesystem_method', $method, $args, $context, $allow_relaxed_file_ownership );
 }
 
 /**
- * Displays a form to the user to request for their FTP/SSH details in order to connect to the filesystem.
+ * Displays a form to the user to request for their FTP/SSH details in order
+ * to connect to the filesystem.
+ *
  * All chosen/entered details are saved, Excluding the Password.
  *
- * Hostnames may be in the form of hostname:portnumber (eg: wordpress.org:2467) to specify an alternate FTP/SSH port.
+ * Hostnames may be in the form of hostname:portnumber (eg: wordpress.org:2467)
+ * to specify an alternate FTP/SSH port.
  *
- * Plugins may override this form by returning true|false via the <code>request_filesystem_credentials</code> filter.
+ * Plugins may override this form by returning true|false via the
+ * {@see 'request_filesystem_credentials'} filter.
  *
- * @since 2.5.0
+ * @since 2.5.
+ *
+ * @todo Properly mark optional arguments as such
  *
  * @param string $form_post the URL to post the form to
  * @param string $type the chosen Filesystem method in use
  * @param boolean $error if the current request has failed to connect
  * @param string $context The directory which is needed access to, The write-test will be performed on this directory by get_filesystem_method()
  * @param string $extra_fields Extra POST fields which should be checked for to be included in the post.
+ * @param bool $allow_relaxed_file_ownership Whether to allow Group/World writable.
  * @return boolean False on failure. True on success.
  */
-function request_filesystem_credentials($form_post, $type = '', $error = false, $context = false, $extra_fields = null) {
+function request_filesystem_credentials($form_post, $type = '', $error = false, $context = false, $extra_fields = null, $allow_relaxed_file_ownership = false ) {
 
 	/**
 	 * Filter the filesystem credentials form output.
@@ -958,14 +999,16 @@ function request_filesystem_credentials($form_post, $type = '', $error = false,
 	 *                             Default false.
 	 * @param string $context      Full path to the directory that is tested for
 	 *                             being writable.
+	 * @param bool $allow_relaxed_file_ownership Whether to allow Group/World writable.
 	 * @param array  $extra_fields Extra POST fields.
 	 */
-	$req_cred = apply_filters( 'request_filesystem_credentials', '', $form_post, $type, $error, $context, $extra_fields );
+	$req_cred = apply_filters( 'request_filesystem_credentials', '', $form_post, $type, $error, $context, $extra_fields, $allow_relaxed_file_ownership );
 	if ( '' !== $req_cred )
 		return $req_cred;
 
-	if ( empty($type) )
-		$type = get_filesystem_method(array(), $context);
+	if ( empty($type) ) {
+		$type = get_filesystem_method( array(), $context, $allow_relaxed_file_ownership );
+	}
 
 	if ( 'direct' == $type )
 		return true;
@@ -1028,7 +1071,9 @@ function request_filesystem_credentials($form_post, $type = '', $error = false,
 			$stored_credentials['hostname'] .= ':' . $stored_credentials['port'];
 
 		unset($stored_credentials['password'], $stored_credentials['port'], $stored_credentials['private_key'], $stored_credentials['public_key']);
-		update_option('ftp_credentials', $stored_credentials);
+		if ( ! defined( 'WP_INSTALLING' ) ) {
+			update_option( 'ftp_credentials', $stored_credentials );
+		}
 		return $credentials;
 	}
 	$hostname = isset( $credentials['hostname'] ) ? $credentials['hostname'] : '';
